import { Gitlab, GitlabService } from "@egg/net-tool"
import { RecordModel } from "pocketbase"

import { Context } from "../../types"
import { managePbError } from "../../utils/pbTools"
import pbClient from "../pbClient"

const DB_NAME = "gitlabProject"

// Gitlab项目接口定义
export interface GitlabProject {
  projectId: number
  name: string
  desc: string
  pathWithNamespace: string
  webUrl: string
  openCICDNotify: boolean
  openMRSummary: boolean
}

// Gitlab项目模型类型
export type GitlabProjectModel = GitlabProject & RecordModel

/**
 * 创建Gitlab项目
 * @param {GitlabProject} project - Gitlab项目信息
 * @returns {Promise<GitlabProjectModel>} - 创建的Gitlab项目模型
 */
const create = async (project: GitlabProject) =>
  managePbError<GitlabProjectModel>(() =>
    pbClient.collection(DB_NAME).create(project)
  )

/**
 * 通过项目ID获取Gitlab项目
 * @param {number} projectId - 项目ID
 * @returns {Promise<GitlabProjectModel | null>} - Gitlab项目模型或null
 */
const getByProjectId = async (projectId: number) =>
  managePbError<GitlabProjectModel>(() =>
    pbClient.collection(DB_NAME).getFirstListItem(`projectId = "${projectId}"`)
  )

/**
 * 通过项目ID获取Gitlab项目，如果不存在则创建
 * @param {number} projectId - 项目ID
 * @param {GitlabService} gitlabService - Gitlab服务
 * @returns {Promise<GitlabProjectModel | null>} - Gitlab项目模型或null
 */
const getAndCreate = async (
  projectId: number,
  gitlabService: GitlabService
) => {
  const project = await getByProjectId(projectId)
  if (project) return project
  gitlabService.setProjectId(projectId)
  const projectInfo = await gitlabService.project.getDetail()
  if (!projectInfo) return null
  const { name, description, path_with_namespace, web_url } = projectInfo
  const newProject = {
    projectId,
    name,
    desc: description,
    pathWithNamespace: path_with_namespace,
    webUrl: web_url,
    openCICDNotify: false,
    openMRSummary: false,
  }
  return await create(newProject)
}

/**
 * 通过上下文获取Gitlab项目
 * @param {Context} context - 上下文对象
 * @returns {Promise<GitlabProjectModel | null>} - Gitlab项目模型或null
 */
const getByCtx = async ({ body: rawBody, gitlabService }: Context) => {
  const body = rawBody as Gitlab.PipelineEvent | Gitlab.MergeRequestEvent
  const projectId = body.project.id
  if (!projectId) return null
  return await getAndCreate(projectId, gitlabService)
}

const update = async (id: string, project: Partial<GitlabProjectModel>) =>
  managePbError<GitlabProjectModel>(() =>
    pbClient.collection(DB_NAME).update(id, project)
  )

const gitlabProject = {
  create,
  update,
  getByProjectId,
  getByCtx,
  getAndCreate,
}

export default gitlabProject
